import Flutter
import UIKit
import Passbase

private struct InitializeArguments {
    let apiKey: String
    let additionalArguments: [String: String]?
    let prefillUserEmail: String?
    
    init?(json: Any?) {
        guard let json = json as? [String: Any],
            let apiKey = json["apiKey"] as? String else {
                return nil
        }
        
        self.apiKey = apiKey
        self.additionalArguments = json["additionalArguments"] as? [String: String]
        self.prefillUserEmail = json["prefillUserEmail"] as? String
    }
}

private enum FlutterBridgeMethod: String {
    case initialize
    case startVerification
    case setActionButtonBgColor
    case setActionButtonDeactivatedBgColor
    case setActionButtonTextColor
    case setDisclaimerTextColor
    case setTitleTextColor
    case setSubtitleTextColor
    case setLoadingIndicatorColor
}

public class SwiftPassbaseFlutterPlugin: NSObject, FlutterPlugin {
    private var verificationResult: FlutterResult?
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "passbase_flutter",
                                           binaryMessenger: registrar.messenger())
        let instance = SwiftPassbaseFlutterPlugin()
        registrar.addMethodCallDelegate(instance,
                                        channel: channel)
    }
    
    // MARK: - Call handling
    public func handle(_ call: FlutterMethodCall,
                       result: @escaping FlutterResult) {
        guard let method = FlutterBridgeMethod(rawValue: call.method) else {
            result("Method is not implemented")
            return
        }
        
        switch method {
        case .initialize:
            if let arguments = InitializeArguments(json: call.arguments) {
                Passbase.additionalAttributes = arguments.additionalArguments
                Passbase.prefillUserEmail = arguments.prefillUserEmail
                Passbase.initialize(publishableApiKey: arguments.apiKey)
                result(nil)
            } else {
                result("Can't get arguments")
            }
        case .startVerification:
            guard let viewController = UIApplication.shared.keyWindow?.rootViewController else {
                result("Can't get root view controller")
                return
            }
            do {
                Passbase.delegate = self
                try Passbase.startVerification(from: viewController)
                verificationResult = result
            } catch {
                result(error.localizedDescription)
            }
        default:
            if let arguments = call.arguments as? [Int],
                let color = getColor(from: arguments) {
                switch method {
                case .setActionButtonBgColor:
                    Passbase.buttonUI.actionButtonBgColor = color
                case .setActionButtonDeactivatedBgColor:
                    Passbase.buttonUI.actionButtonDeactivatedBgColor = color
                case .setActionButtonTextColor:
                    Passbase.buttonUI.actionButtonTextColor = color
                case .setDisclaimerTextColor:
                    Passbase.buttonUI.disclaimerTextColor = color
                case .setTitleTextColor:
                    Passbase.buttonUI.titleTextColor = color
                case .setSubtitleTextColor:
                    Passbase.buttonUI.subtitleTextColor = color
                case .setLoadingIndicatorColor:
                    Passbase.buttonUI.loadingIndicatorColor = color
                default:
                    result("Can't get provided color")
                    return
                }
                result(nil)
            } else {
                result("Can't get provided color")
            }
        }
    }
    
    private func getColor(from arguments: [Int]) -> UIColor? {
        guard arguments.count >= 4 else {
            return nil
        }
        
        let r = arguments[0]
        let g = arguments[1]
        let b = arguments[2]
        let a = arguments[3]
        
        return UIColor(red: CGFloat(r) / 255,
                       green: CGFloat(g) / 255,
                       blue: CGFloat(b) / 255,
                       alpha: CGFloat(a) / 255)
    }
}

// MARK: - PassbaseDelegate
extension SwiftPassbaseFlutterPlugin: PassbaseDelegate {
    public func didCompletePassbaseVerification(authenticationKey: String) {
        verificationResult?(authenticationKey)
        verificationResult = nil
    }
    
    public func didCancelPassbaseVerification() {
        verificationResult?(FlutterError(code: "CANCEL",
                                         message: "Verification canceled",
                                         details: nil))
        verificationResult = nil
    }
}
